'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _findRoot = require('find-root');

var _findRoot2 = _interopRequireDefault(_findRoot);

var _webpackSources = require('webpack-sources');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function cleanPath(path) {
  return path.split('/node_modules/').join('/~/');
}

// Get closest package definition from path
function getClosestPackage(modulePath) {
  var root;
  var pkg;

  // Catch findRoot or require errors
  try {
    root = (0, _findRoot2.default)(modulePath);
    pkg = require(_path2.default.join(root, 'package.json'));
  } catch (e) {
    return null;
  }

  // If the package.json does not have a name property, try again from
  // one level higher.
  // https://github.com/jsdnxx/find-root/issues/2
  // https://github.com/date-fns/date-fns/issues/264#issuecomment-265128399
  if (!pkg.name) {
    return getClosestPackage(_path2.default.resolve(root, '..'));
  }

  return {
    package: pkg,
    path: root
  };
}

function check(compilation, modulesToCheck) {
  var context = compilation.compiler.context;
  var modules = {};

  function cleanPathRelativeToContext(modulePath) {
    var cleanedPath = cleanPath(modulePath);

    // Make relative to compilation context
    if (cleanedPath.indexOf(context) === 0) {
      cleanedPath = '.' + cleanedPath.replace(context, '');
    }

    return cleanedPath;
  }

  compilation.modules.forEach(function (module) {
    if (!module.resource) {
      return;
    }

    var pkg;
    var packagePath;

    var closestPackage = getClosestPackage(module.resource);

    // Skip module if no closest package is found
    if (!closestPackage) {
      return;
    }

    pkg = closestPackage.package;
    packagePath = closestPackage.path;

    var modulePath = cleanPathRelativeToContext(packagePath);

    var version = pkg.version;

    if (modulesToCheck.indexOf(pkg.name) < 0) {
      return;
    }

    modules[pkg.name] = modules[pkg.name] || [];

    var isSeen = false;

    modules[pkg.name].forEach(function (module) {
      if (module.version === version) {
        isSeen = true;
      }
    });

    if (!isSeen) {
      var entry = { version: version, path: modulePath };

      modules[pkg.name].push(entry);
    }
  });

  var duplicates = {};
  Object.keys(modules).forEach(function (name) {
    if (modules[name].length > 1) {
      duplicates[name] = modules[name];
    }
  });

  return duplicates;
}

function formatMsg(duplicates) {
  var error = 'Duplicate (conflicting) packages loaded, make sure to use only one: ';

  if (Object.keys(duplicates).length) {
    Object.keys(duplicates).forEach(function (key) {
      var instances = duplicates[key];
      instances = instances.map(function (version) {
        var str = version.version + ' ' + version.path;
        return str;
      });
      error += '\\n  <' + key + '> \\n';
      error += '    ' + instances.join('\\n    ') + '\\n';
    });
  }

  return error;
}

var DuplicateChecker = function () {
  function DuplicateChecker(options) {
    _classCallCheck(this, DuplicateChecker);

    this.options = Object.assign({}, options);;
  }

  _createClass(DuplicateChecker, [{
    key: 'apply',
    value: function apply(compiler) {
      var modulesToCheck = this.options.modulesToCheck;
      if (!modulesToCheck || !modulesToCheck.length) {
        return;
      }

      // Webpack 4
      if (compiler.hooks && compiler.hooks.compilation && compiler.hooks.compilation.tap) {
        compiler.hooks.compilation.tap('RaxDuplicateCheckerPlugin', function (compilation) {
          var duplicates = check(compilation, modulesToCheck);
          if (!Object.keys(duplicates).length) {
            return;
          }

          var errorMessages = 'console.error(\'' + formatMsg(duplicates) + '\');';

          compilation.hooks.optimizeChunkAssets.tap('RaxDuplicateCheckerPlugin', function (chunks) {
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
              for (var _iterator = chunks[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                var chunk = _step.value;

                // Entry only
                if (!chunk.canBeInitial()) {
                  continue;
                }

                chunk.files.forEach(function (file) {
                  compilation.assets[file] = new _webpackSources.ConcatSource(compilation.assets[file], '\n', errorMessages);
                });
              }
            } catch (err) {
              _didIteratorError = true;
              _iteratorError = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion && _iterator.return) {
                  _iterator.return();
                }
              } finally {
                if (_didIteratorError) {
                  throw _iteratorError;
                }
              }
            }
          });
        });
      } else {
        compiler.plugin('compilation', function (compilation) {
          var duplicates = check(compilation, modulesToCheck);
          if (!Object.keys(duplicates).length) {
            return;
          }

          var errorMessages = 'console.error(\'' + formatMsg(duplicates) + '\');';

          compilation.plugin('optimize-chunk-assets', function (chunks) {
            chunks.forEach(function (chunk) {
              // Entry only
              try {
                // In webpack2 chunk.initial was removed. Use isInitial()
                if (!chunk.initial) return;
              } catch (e) {
                if (!chunk.isInitial()) return;
              }

              chunk.files.forEach(function (file) {
                compilation.assets[file] = new _webpackSources.ConcatSource(compilation.assets[file], '\n', errorMessages);
              });
            });
          });
        });
      }
    }
  }]);

  return DuplicateChecker;
}();

exports.default = DuplicateChecker;
module.exports = exports['default'];